@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -124,14 +124,10 @@ ItemText::ItemText(const QString &text, const QString &richText, int maxLines, i
 
     setReadOnly(true);
     setUndoRedoEnabled(false);
-    setTextInteractionFlags(
-                textInteractionFlags() | Qt::LinksAccessibleByMouse);
 
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setFrameStyle(QFrame::NoFrame);
 
-    setContextMenuPolicy(Qt::NoContextMenu);
-
     if ( !richText.isEmpty() ) {
         m_textDocument.setHtml(richText);
         // Use plain text instead if rendering HTML fails or result is empty.
@@ -141,8 +137,6 @@ ItemText::ItemText(const QString &text, const QString &richText, int maxLines, i
     if (!m_isRichText)
         m_textDocument.setPlainText(text);
 
-    m_textDocument.setDocumentMargin(0);
-
     if (maxLines > 0) {
         QTextBlock block = m_textDocument.findBlockByLineNumber(maxLines);
         if (block.isValid()) {
@@ -214,7 +208,12 @@ void ItemText::highlight(const QRegularExpression &re, const QFont &highlightFon
 
 void ItemText::updateSize(QSize maximumSize, int idealWidth)
 {
-    const int scrollBarWidth = verticalScrollBar()->isVisible() ? verticalScrollBar()->width() : 0;
+    if ( m_textDocument.isEmpty() ) {
+        setFixedSize(0, 0);
+        return;
+    }
+
+    const int scrollBarWidth = verticalScrollBar()->width();
     setMaximumHeight( maximumSize.height() );
     setFixedWidth(idealWidth);
     m_textDocument.setTextWidth(idealWidth - scrollBarWidth);
@@ -231,9 +230,6 @@ void ItemText::updateSize(QSize maximumSize, int idealWidth)
     if (document() != &m_textDocument)
         setDocument(&m_textDocument);
 
-    const QRectF rect = m_textDocument.documentLayout()->frameBoundingRect(m_textDocument.rootFrame());
-    setFixedWidth( static_cast<int>(rect.right()) );
-
     QTextCursor tc(&m_textDocument);
     tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
     const auto h = static_cast<int>( cursorRect(tc).bottom() + 2 * logicalDpiY() / 96.0 );
@@ -310,9 +306,14 @@ ItemWidget *ItemTextLoader::create(const QVariantMap &data, QWidget *parent, boo
     text = normalizeText(text);
 
     ItemText *item = nullptr;
+    Qt::TextInteractionFlags interactionFlags(Qt::LinksAccessibleByMouse);
     // Always limit text size for performance reasons.
     if (preview) {
         item = new ItemText(text, richText, maxLineCountInPreview, maxLineLengthInPreview, 0, parent);
+        item->setFocusPolicy(Qt::StrongFocus);
+        interactionFlags = interactionFlags
+                | Qt::TextSelectableByKeyboard
+                | Qt::LinksAccessibleByKeyboard;
     } else {
         int maxLines = m_settings.value(optionMaximumLines, maxLineCount).toInt();
         if (maxLines <= 0 || maxLines > maxLineCount)
@@ -320,7 +321,9 @@ ItemWidget *ItemTextLoader::create(const QVariantMap &data, QWidget *parent, boo
         const int maxHeight = m_settings.value(optionMaximumHeight, 0).toInt();
         item = new ItemText(text, richText, maxLines, maxLineLength, maxHeight, parent);
         item->viewport()->installEventFilter(item);
+        item->setContextMenuPolicy(Qt::NoContextMenu);
     }
+    item->setTextInteractionFlags(item->textInteractionFlags() | interactionFlags);
 
     return item;
 }
