@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,6 +20,7 @@
 #include "clipboardserver.h"
 
 #include "common/action.h"
+#include "common/appconfig.h"
 #include "common/clientsocket.h"
 #include "common/client_server.h"
 #include "common/commandstatus.h"
@@ -44,6 +45,7 @@
 #include <QMessageBox>
 #include <QMimeData>
 #include <QSessionManager>
+#include <QTextEdit>
 
 #ifdef NO_GLOBAL_SHORTCUTS
 class QxtGlobalShortcut final {};
@@ -67,6 +69,24 @@ uint monitorCommandStateHash(const QVector<Command> &commands)
     return hash;
 }
 
+void setTabWidth(QTextEdit *editor, int spaces)
+{
+    const QLatin1Char space(' ');
+    const QFontMetricsF metrics(editor->fontMetrics());
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,11,0)
+    const qreal width = metrics.horizontalAdvance(QString(spaces, space));
+#else
+    const qreal width = metrics.width(space) * spaces;
+#endif
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,10,0)
+    editor->setTabStopDistance(width);
+#else
+    editor->setTabStopWidth(width);
+#endif
+}
+
 } // namespace
 
 ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
@@ -76,7 +96,7 @@ ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
     , m_shortcutActions()
     , m_ignoreKeysTimer()
 {
-    setCurrentThreadName("Server");
+    setLogLabel("Server");
 
     const QString serverName = clipboardServerName();
     m_server = new Server(serverName, this);
@@ -93,6 +113,13 @@ ClipboardServer::ClipboardServer(QApplication *app, const QString &sessionName)
         return;
     }
 
+    if ( sessionName.isEmpty() ) {
+        QGuiApplication::setApplicationDisplayName("CopyQ");
+    } else {
+        QGuiApplication::setApplicationDisplayName(
+            QString("CopyQ-%1").arg(sessionName));
+    }
+
     QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps, true);
 
     QApplication::setQuitOnLastWindowClosed(false);
@@ -529,9 +556,28 @@ bool ClipboardServer::eventFilter(QObject *object, QEvent *ev)
                 else
                     m_wnd->enterSearchMode();
             }
+        // Omit overriding arrow keys in text editors.
+        } else if ( type == QEvent::ShortcutOverride &&
+                    object->property("textInteractionFlags")
+                    .value<Qt::TextInteractionFlags>()
+                    .testFlag(Qt::TextSelectableByKeyboard) )
+        {
+            QKeyEvent *keyevent = static_cast<QKeyEvent *>(ev);
+            if ( keyevent->key() == Qt::Key_Left
+                 || keyevent->key() == Qt::Key_Right
+                 || keyevent->key() == Qt::Key_Up
+                 || keyevent->key() == Qt::Key_Down)
+            {
+                ev->accept();
+                return true;
+            }
         }
     } else if (type == QEvent::Paint) {
         setActivePaintDevice(object);
+    } else if (type == QEvent::FontChange) {
+        QTextEdit *editor = qobject_cast<QTextEdit*>(object);
+        if (editor)
+            setTabWidth(editor, m_textTabSize);
     }
 
     return false;
@@ -539,6 +585,8 @@ bool ClipboardServer::eventFilter(QObject *object, QEvent *ev)
 
 void ClipboardServer::loadSettings()
 {
+    m_textTabSize = AppConfig().option<Config::text_tab_width>();
+
     if (m_monitor) {
         stopMonitoring();
         startMonitoring();
