@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -144,6 +144,18 @@ public:
         return refresh() ? m_dataGuard->urls() : QList<QUrl>();
     }
 
+    QString text()
+    {
+        ElapsedGuard _("text");
+        return refresh() ? m_dataGuard->text() : QString();
+    }
+
+    bool hasText()
+    {
+        ElapsedGuard _("hasText");
+        return refresh() && m_dataGuard->hasText();
+    }
+
     QImage getImageData()
     {
         ElapsedGuard _("imageData");
@@ -174,6 +186,9 @@ public:
             return bytes;
         }
 
+        if ( format == mimeText && !hasFormat(mimeText) )
+            return text().toUtf8();
+
         if ( format.startsWith("text/") )
             return dataToText( data(format), format ).toUtf8();
 
@@ -355,7 +370,7 @@ QVariantMap cloneData(const QMimeData &rawData, QStringList formats, bool *abort
      Images in SVG and other XML formats are expected to be relatively small
      so these doesn't have to be ignored.
      */
-    if ( formats.contains(mimeText) && data.hasFormat(mimeText) ) {
+    if ( formats.contains(mimeText) && data.hasText() ) {
         const QString mimeImagePrefix = "image/";
         const auto first = std::remove_if(
                     std::begin(formats), std::end(formats),
@@ -407,6 +422,19 @@ QVariantMap cloneData(const QMimeData &data)
             formats.append(mime);
     }
 
+    if ( !formats.contains(mimeText) ) {
+        const QString textPrefix(QLatin1String(mimeText) + ";");
+        bool containsText = false;
+        for (int i = formats.size() - 1; i >= 0; --i) {
+            if ( formats[i].startsWith(textPrefix) ) {
+                formats.removeAt(i);
+                containsText = true;
+            }
+        }
+        if (containsText)
+            formats.append(mimeText);
+    }
+
     return cloneData(data, formats);
 }
 
@@ -442,19 +470,37 @@ bool anySessionOwnsClipboardData(const QVariantMap &data)
 QString elideText(const QString &text, const QFont &font, const QString &format,
                   bool escapeAmpersands, int maxWidthPixels, int maxLines)
 {
+    if ( text.isEmpty() )
+        return QString();
+
     if (maxWidthPixels <= 0)
         maxWidthPixels = smallIconSize() * 20;
 
     QStringList lines = text.split('\n');
 
-    // Ignore empty lines at beginning.
-    static const QRegularExpression reNonEmpty(".*\\S.*");
-    const int firstLine = qMax(0, lines.indexOf(reNonEmpty));
-    const int lastLine = qMax(0, lines.lastIndexOf(reNonEmpty, firstLine + maxLines - 1));
+    int firstLine = -1;
+    int lastLine = -1;
+    int commonIndent = text.size();
+    static const QRegularExpression reNonSpace("\\S");
+    for (int i = 0; i < lines.size(); ++i) {
+        const auto &line = lines[i];
+        const int lineIndent = line.indexOf(reNonSpace);
+        if (lineIndent == -1)
+            continue;
 
-    // If empty lines are at beginning, prepend triple dot.
-    if (firstLine != 0)
-        lines[firstLine].prepend("...");
+        if (firstLine == -1)
+            firstLine = i;
+
+        lastLine = i;
+        if (lineIndent < commonIndent)
+            commonIndent = lineIndent;
+
+        if (firstLine - lastLine + 1 >= maxLines)
+            break;
+    }
+
+    if (lastLine == -1)
+        return QString("...");
 
     // If there are too many lines, append triple dot.
     if (lastLine + 1 != lines.size())
@@ -470,19 +516,9 @@ QString elideText(const QString &text, const QFont &font, const QString &format,
 #endif
 
     // Remove redundant spaces from single line text.
-    if (lines.size() == 1)
+    if (lines.size() == 1) {
         lines[0] = lines[0].simplified();
-
-    // Find common indentation.
-    int commonIndent = lines.value(0).size();
-    static const QRegularExpression reNonSpace("\\S");
-    for (const auto &line : lines) {
-        const int lineIndent = line.indexOf(reNonSpace);
-        if (lineIndent != -1 && lineIndent < commonIndent) {
-            commonIndent = lineIndent;
-            if (commonIndent == 0)
-                break;
-        }
+        commonIndent = 0;
     }
 
     // Remove common indentation each line and elide text if too long.
@@ -496,6 +532,14 @@ QString elideText(const QString &text, const QFont &font, const QString &format,
         line = fm.elidedText(line, Qt::ElideMiddle, maxWidthPixels - formatWidth);
     }
 
+    // If empty lines are at beginning, prepend triple dot.
+    if (firstLine != 0) {
+        if (lines.size() == 1)
+            lines.first().prepend("...");
+        else
+            lines.prepend("...");
+    }
+
     QString result = lines.join("\n");
 
     // Escape all ampersands.
@@ -609,6 +653,9 @@ bool handleViKey(QKeyEvent *event, QObject *eventReceiver)
             return false;
         }
         break;
+    case Qt::Key_X:
+        key = Qt::Key_Delete;
+        break;
     case Qt::Key_BracketLeft:
         if (mods & Qt::ControlModifier) {
             key = Qt::Key_Escape;
@@ -665,7 +712,7 @@ QByteArray makeClipboardOwnerData()
 {
     static int id = 0;
     return qgetenv("COPYQ_SESSION_NAME")
-           + " " + currentThreadLabel()
+           + " " + logLabel()
            + "/" + QByteArray::number(++id);
 }
 
