@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -460,7 +460,7 @@ void ClipboardBrowser::setEditorWidget(ItemEditorWidget *editor, bool changeClip
             if ( oldSearch.isEmpty() )
                 emit searchHideRequest();
             else
-                emit searchShowRequest(d.searchExpression().pattern());
+                emit searchShowRequest(oldSearch);
         }
 
         emit internalEditorStateChanged(this);
@@ -576,9 +576,7 @@ void ClipboardBrowser::connectModelAndDelegate()
 
 void ClipboardBrowser::updateItemMaximumSize()
 {
-    QSize minSize = viewport()->contentsRect().size();
-    if (verticalScrollBarPolicy() != Qt::ScrollBarAlwaysOff)
-         minSize -= QSize(verticalScrollBar()->width(), 0);
+    const QSize minSize = viewport()->contentsRect().size();
     if ( minSize.width() > 0 )
         d.setItemSizes(m_sharedData->textWrap ? minSize : QSize(2048, 2048), minSize.width());
 }
@@ -643,7 +641,7 @@ void ClipboardBrowser::preload(int pixels, bool above, const QModelIndex &start)
 
     const auto margins = m_sharedData->theme.margins();
     const auto rowNumberSize = m_sharedData->theme.rowNumberSize();
-    const auto padding = QPoint(rowNumberSize.width() + margins.width() - s, margins.height() - s);
+    const auto padding = QPoint(rowNumberSize.width() + margins.width() - s, margins.height());
 
     int items = 0;
     bool anyShown = false;
@@ -810,44 +808,6 @@ int ClipboardBrowser::removeIndexes(const QModelIndexList &indexes, QString *err
     return dropIndexes(indexes);
 }
 
-void ClipboardBrowser::paste(const QVariantMap &data, int destinationRow)
-{
-    if ( !isLoaded() ) {
-        loadItems();
-        if ( !isLoaded() )
-            return;
-    }
-
-    // Insert items from clipboard or just clipboard content.
-    if ( data.contains(mimeItems) ) {
-        const QByteArray bytes = data[mimeItems].toByteArray();
-        QDataStream stream(bytes);
-
-        QList<QVariantMap> dataList;
-        while ( !stream.atEnd() ) {
-            QVariantMap dataMap;
-            stream >> dataMap;
-            dataList.append(dataMap);
-        }
-
-        // list size limit
-        if ( !allocateSpaceForNewItems(dataList.size()) ) {
-            QMessageBox::information(
-                        this, tr("Cannot Add New Items"),
-                        tr("Tab is full. Failed to remove any items.") );
-            return;
-        }
-
-        // create new item
-        const int newRow = destinationRow < 0 ? m.rowCount() : qMin(destinationRow, m.rowCount());
-        m.insertItems(dataList, newRow);
-    } else {
-        add(data, destinationRow);
-    }
-
-    saveItems();
-}
-
 QPixmap ClipboardBrowser::renderItemPreview(const QModelIndexList &indexes, int maxWidth, int maxHeight)
 {
     int h = 0;
@@ -979,15 +939,8 @@ void ClipboardBrowser::contextMenuEvent(QContextMenuEvent *event)
 void ClipboardBrowser::resizeEvent(QResizeEvent *event)
 {
     QListView::resizeEvent(event);
-
     // WORKAROUND: Omit calling resizeEvent() recursively.
-    if (m_resizing) {
-        m_timerUpdateSizes.start();
-    } else {
-        m_resizing = true;
-        updateSizes();
-        m_resizing = false;
-    }
+    m_timerUpdateSizes.start();
 }
 
 void ClipboardBrowser::showEvent(QShowEvent *event)
@@ -1077,7 +1030,7 @@ void ClipboardBrowser::dropEvent(QDropEvent *event)
         return; // handled in mouseMoveEvent()
 
     const QVariantMap data = cloneData( *event->mimeData() );
-    paste(data, m_dragTargetRow);
+    add(data, m_dragTargetRow);
     m_dragTargetRow = -1;
 }
 
@@ -1314,13 +1267,6 @@ bool ClipboardBrowser::openEditor(const QModelIndex &index)
     return false;
 }
 
-void ClipboardBrowser::addItems(const QStringList &items)
-{
-    for(int i=items.count()-1; i>=0; --i) {
-        add(items[i]);
-    }
-}
-
 void ClipboardBrowser::editNotes()
 {
     QModelIndex ind = currentIndex();
@@ -1619,15 +1565,20 @@ bool ClipboardBrowser::allocateSpaceForNewItems(int newItemCount)
         return true;
 
     QModelIndexList indexesToRemove;
-    QString error;
     for (int row = m.rowCount() - 1; row >= 0 && indexesToRemove.size() < toRemove; --row) {
         const auto index = m.index(row);
-        if ( m_itemSaver->canRemoveItems(QModelIndexList() << index, &error) )
+        if ( m_itemSaver->canDropItem(index) )
             indexesToRemove.append(index);
     }
 
-    if (indexesToRemove.size() < toRemove)
+    if (indexesToRemove.size() < toRemove) {
+        log( QString("Cannot add new items. Tab \"%1\" reached the maximum number of items.")
+             .arg(m_tabName), LogWarning );
+        emit error(
+            tr("Cannot add new items to tab %1. Please remove items manually to make space.")
+            .arg(quoteString(m_tabName)) );
         return false;
+    }
 
     dropIndexes(indexesToRemove);
     return true;
@@ -1642,22 +1593,37 @@ bool ClipboardBrowser::add(const QVariantMap &data, int row)
 {
     if ( !isLoaded() ) {
         loadItems();
-        if ( !isLoaded() )
+        if ( !isLoaded() ) {
+            log( QString("Cannot add new items. Tab %1 is not loaded.").arg(m_tabName), LogWarning );
             return false;
+        }
     }
 
-    // list size limit
-    if ( !allocateSpaceForNewItems(1) ) {
-        QMessageBox::information(
-                    this, tr("Cannot Add New Items"),
-                    tr("Tab is full. Failed to remove any items.") );
-        return false;
-    }
-
-    // create new item
     const int newRow = row < 0 ? m.rowCount() : qMin(row, m.rowCount());
-    m.insertItem(data, newRow);
 
+    if ( data.contains(mimeItems) ) {
+        const QByteArray bytes = data[mimeItems].toByteArray();
+        QDataStream stream(bytes);
+
+        QList<QVariantMap> dataList;
+        while ( !stream.atEnd() ) {
+            QVariantMap dataMap;
+            stream >> dataMap;
+            dataList.append(dataMap);
+        }
+
+        if ( !allocateSpaceForNewItems(dataList.size()) )
+            return false;
+
+        m.insertItems(dataList, newRow);
+    } else {
+        if ( !allocateSpaceForNewItems(1) )
+            return false;
+
+        m.insertItem(data, newRow);
+    }
+
+    saveItems();
     return true;
 }
 
@@ -1684,15 +1650,13 @@ void ClipboardBrowser::addUnique(const QVariantMap &data, ClipboardMode mode)
             const auto oldText = getTextData(previousData);
             if ( (mode == ClipboardMode::Clipboard)
                  ? (newText == oldText)
-                 : getTextData(data).contains(getTextData(previousData)) )
+                 : newText.contains(oldText) )
             {
                 COPYQ_LOG("New item: Merging with top item");
 
-                const QSet<QString> formatsToAdd = previousData.keys().toSet() - data.keys().toSet();
-
-                auto newData = data;
-                for (const auto &format : formatsToAdd)
-                    newData.insert(format, previousData[format]);
+                auto newData = previousData;
+                for (auto it = data.constBegin(); it != data.constEnd(); ++it)
+                    newData.insert(it.key(), it.value());
 
                 m.setData(firstIndex, newData, contentType::data);
 
@@ -1759,8 +1723,15 @@ void ClipboardBrowser::delayedSaveItems()
 
 void ClipboardBrowser::updateSizes()
 {
+    if (m_resizing) {
+        m_timerUpdateSizes.start();
+        return;
+    }
+
+    m_resizing = true;
     updateItemMaximumSize();
     updateEditorGeometry();
+    m_resizing = false;
 }
 
 void ClipboardBrowser::updateCurrent()
@@ -1843,6 +1814,11 @@ void ClipboardBrowser::move(int key)
     scrollTo( currentIndex() );
 }
 
+void ClipboardBrowser::move(const QModelIndexList &indexes, int targetRow)
+{
+    moveIndexes( indexes, targetRow, &m, MoveType::Absolute );
+}
+
 QWidget *ClipboardBrowser::currentItemPreview(QWidget *parent)
 {
     if (!isLoaded())
@@ -1900,3 +1876,8 @@ bool ClipboardBrowser::maybeCloseEditors()
 
     return true;
 }
+
+void ClipboardBrowser::keyboardSearch(const QString &text)
+{
+    emit searchShowRequest(text);
+}
