@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -109,7 +109,7 @@ class ClipboardBrowser final : public QListView
         /**
          * Override to disable default QAbstractItemView search.
          */
-        void keyboardSearch(const QString &) override {}
+        void keyboardSearch(const QString &text) override;
 
         QVariantMap copyIndex(const QModelIndex &index) const;
 
@@ -118,9 +118,6 @@ class ClipboardBrowser final : public QListView
         /** Remove items and return smallest row number (new current item if selection was removed). */
         int removeIndexes(const QModelIndexList &indexes, QString *error = nullptr);
 
-        /** Paste items. */
-        void paste(const QVariantMap &data, int destinationRow);
-
         /** Render preview image with items. */
         QPixmap renderItemPreview(const QModelIndexList &indexes, int maxWidth, int maxHeight);
 
@@ -162,8 +159,6 @@ class ClipboardBrowser final : public QListView
         bool openEditor(const QByteArray &textData, bool changeClipboard = false);
         /** Open editor for an item. */
         bool openEditor(const QModelIndex &index);
-        /** Add items. */
-        void addItems(const QStringList &items);
 
         /** Set current item. */
         void setCurrent(int row, bool keepSelection = false, bool setCurrentOnly = false);
@@ -192,6 +187,8 @@ class ClipboardBrowser final : public QListView
 
         void move(int key);
 
+        void move(const QModelIndexList &indexes, int targetRow);
+
         QWidget *currentItemPreview(QWidget *parent);
 
         void findNext();
