@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -475,6 +475,7 @@ private:
 
     void updateItemPreviewTimeout();
 
+    void toggleItemPreviewVisible();
     void setItemPreviewVisible(bool visible);
 
     /** Update icon snip animation. */
@@ -511,6 +512,8 @@ private:
 
     void onActionDialogAccepted(const Command &command, const QStringList &arguments, const QVariantMap &data);
 
+    void onSearchShowRequest(const QString &text);
+
     void updateEnabledCommands();
 
     void updateCommands(QVector<Command> allCommands, bool forceSave);
@@ -587,7 +590,7 @@ private:
     bool closeMinimizes() const;
 
     template <typename SlotReturnType>
-    QAction *createAction(int id, MainWindowActionSlot<SlotReturnType> slot, QMenu *menu);
+    QAction *createAction(int id, MainWindowActionSlot<SlotReturnType> slot, QMenu *menu, QWidget *parent = nullptr);
 
     QAction *addTrayAction(int id);
 
