@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -49,7 +49,7 @@ using ItemSaverPtr = std::shared_ptr<ItemSaverInterface>;
 class ItemScriptableFactoryInterface;
 using ItemScriptableFactoryPtr = std::shared_ptr<ItemScriptableFactoryInterface>;
 
-#define COPYQ_PLUGIN_ITEM_LOADER_ID "com.github.hluk.copyq.itemloader/3.10.0"
+#define COPYQ_PLUGIN_ITEM_LOADER_ID "com.github.hluk.copyq.itemloader/3.11.0"
 
 /**
  * Handles item in list.
@@ -217,6 +217,12 @@ public:
      */
     virtual bool canRemoveItems(const QList<QModelIndex> &indexList, QString *error);
 
+    /**
+     * Called before item is deleted from end so new items can be inserted.
+     * @return true if item can be dropped, false to disallow the drop
+     */
+    virtual bool canDropItem(const QModelIndex &index);
+
     /**
      * Called before items are moved out of list (i.e. deleted) by user.
      * @return true if items can be moved, false to cancel the removal
