@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -102,7 +102,7 @@ private:
 
         ::log( QString("%1: %2")
                .arg(m_label, "Finished in %1 ms")
-               .arg(ms) );
+               .arg(ms), level );
     }
 
     QString m_label;
@@ -1150,6 +1150,19 @@ void Scriptable::remove()
         throwError(error);
 }
 
+void Scriptable::move()
+{
+    m_skipArguments = 1;
+
+    int row;
+    if ( !toInt(argument(0), &row) ) {
+        throwError(argumentError());
+        return;
+    }
+
+    m_proxy->browserMoveSelected(row);
+}
+
 void Scriptable::edit()
 {
     m_skipArguments = -1;
@@ -2012,6 +2025,27 @@ QScriptValue Scriptable::dialog()
     return result;
 }
 
+QScriptValue Scriptable::menuItems()
+{
+    const auto arg = argument(0);
+    if ( arg.isString() ) {
+        m_skipArguments = -1;
+        QVector<QVariantMap> items;
+        for (const auto &arg : arguments())
+            items.append(createDataMap(mimeText, arg));
+        const int i = m_proxy->menuItems(items);
+        if (i == -1 || i >= items.size())
+            return QString();
+        return getTextData(items[i]);
+    }
+
+    m_skipArguments = 1;
+    const auto items = fromScriptValue<QVector<QVariantMap>>( arg, this );
+    if ( items.isEmpty() )
+        return -1;
+    return m_proxy->menuItems(items);
+}
+
 QScriptValue Scriptable::settings()
 {
     m_skipArguments = 2;
@@ -2494,6 +2528,10 @@ void Scriptable::runMenuCommandFilters()
             timer.start();
     });
 
+    // Avoid modifying menu filter data.
+    const int actionId = m_actionId;
+    m_actionId = -1;
+
     connect(&timer, &QTimer::timeout, &loop, [&]() {
         if ( bytes.isEmpty() )
             return;
@@ -2514,18 +2552,14 @@ void Scriptable::runMenuCommandFilters()
             return;
         }
 
-        m_data.clear();
-        if ( !deserializeData(&in, &m_data) ) {
-            log("Failed to deserialize data for menu item filter", LogError);
-            return;
-        }
+        getActionData(actionId);
 
         PerformanceLogger logger( QLatin1String("Menu item filters") );
 
         for (int i = 0; i < matchCommands.length(); ++i) {
             const bool enabled = canExecuteCommandFilter(matchCommands[i]);
-            if ( !m_proxy->enableMenuItem(m_actionId, currentRun, i, enabled) )
-                return;
+            if ( !m_proxy->enableMenuItem(actionId, currentRun, i, enabled) )
+                break;
         }
     });
 
@@ -3280,8 +3314,13 @@ void Scriptable::printError(const QByteArray &message)
 
 void Scriptable::getActionData()
 {
-    if (m_actionId != -1)
-        m_data = m_oldData = m_proxy->getActionData(m_actionId);
+    getActionData(m_actionId);
+}
+
+void Scriptable::getActionData(int actionId)
+{
+    if (actionId != -1)
+        m_data = m_oldData = m_proxy->getActionData(actionId);
 }
 
 void Scriptable::setActionData()
