@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -58,6 +58,7 @@ class Scriptable final : public QObject, protected QScriptable
     Q_PROPERTY(QScriptValue mimeWindowTitle READ getMimeWindowTitle CONSTANT)
     Q_PROPERTY(QScriptValue mimeItems READ getMimeItems CONSTANT)
     Q_PROPERTY(QScriptValue mimeItemNotes READ getMimeItemNotes CONSTANT)
+    Q_PROPERTY(QScriptValue mimeIcon READ getMimeIcon CONSTANT)
     Q_PROPERTY(QScriptValue mimeOwner READ getMimeOwner CONSTANT)
     Q_PROPERTY(QScriptValue mimeClipboardMode READ getMimeClipboardMode CONSTANT)
     Q_PROPERTY(QScriptValue mimeCurrentTab READ getMimeCurrentTab CONSTANT)
@@ -132,6 +133,7 @@ public:
     QScriptValue getMimeWindowTitle() const { return mimeWindowTitle; }
     QScriptValue getMimeItems() const { return mimeItems; }
     QScriptValue getMimeItemNotes() const { return mimeItemNotes; }
+    QScriptValue getMimeIcon() const { return mimeIcon; }
     QScriptValue getMimeOwner() const { return mimeOwner; }
     QScriptValue getMimeClipboardMode() const { return mimeClipboardMode; }
     QScriptValue getMimeCurrentTab() const { return mimeCurrentTab; }
@@ -209,6 +211,7 @@ public slots:
     void insert();
     void remove();
     void edit();
+    void move();
 
     QScriptValue read();
     void write();
@@ -303,6 +306,8 @@ public slots:
 
     QScriptValue dialog();
 
+    QScriptValue menuItems();
+
     QScriptValue settings();
 
     QScriptValue dateString();
@@ -415,6 +420,7 @@ private:
     void printError(const QByteArray &message);
 
     void getActionData();
+    void getActionData(int actionId);
     void setActionData();
 
     QByteArray getClipboardData(const QString &mime, ClipboardMode mode = ClipboardMode::Clipboard);
